@@ -6,11 +6,13 @@ import json as json_util
 from qrunner.utils.config import conf
 from qrunner.utils.log import logger
 
-
-
 IMG = ["jpg", "jpeg", "gif", "bmp", "webp"]
 
 
+class ApiConfig:
+    login_key = ['accessToken', 'signature']
+
+
 def request(func):
     def wrapper(*args, **kwargs):
         func_name = func.__name__
@@ -33,8 +35,14 @@ def request(func):
         auth = kwargs.get("auth", "")
         headers = kwargs.pop("headers", {})
         conf_headers = conf.get_name('api', 'headers')
+        conf_headers_dict = json_util.loads(conf_headers)
+        login_status = kwargs.get('login', True)
         if conf_headers != 'None':
-            headers.update(json_util.loads(conf_headers))
+            if not login_status:
+                for key in ApiConfig.login_key:
+                    conf_headers_dict.pop(key)
+            headers.update(conf_headers_dict)
+            # print(headers)
         kwargs['headers'] = headers
         cookies = kwargs.get("cookies", "")
         params = kwargs.get("params", "")
@@ -85,14 +93,14 @@ class ResponseResult:
 class HttpRequest(object):
 
     @request
-    def get(self, url, params=None, **kwargs):
+    def get(self, url, params=None, login=True, **kwargs):
         base_url = conf.get_name('api', 'base_url')
         if (base_url is not None) and ("http" not in url):
             url = base_url + url
         return requests.get(url, params=params, **kwargs)
 
     @request
-    def post(self, url, data=None, json=None, **kwargs):
+    def post(self, url, data=None, json=None, login=True, **kwargs):
         base_url = conf.get_name('api', 'base_url')
         if (base_url is not None) and ("http" not in url):
             url = base_url + url
